@@ -1,7 +1,17 @@
 /*
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at https://mozilla.org/MPL/2.0/.
+ * Copyright 2021 l1npengtul <l1npengtul@protonmail.com> / The Nokhwa Contributors
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #![deny(clippy::pedantic)]
@@ -15,6 +25,10 @@
 //!
 //! No support or API stability will be given. Subject to change at any time.
 
+#[cfg(all(target_os = "windows", windows))]
+#[macro_use]
+extern crate lazy_static;
+
 use std::{
     borrow::{Borrow, Cow},
     cmp::Ordering,
@@ -356,10 +370,10 @@ pub mod wmf {
         BindingError, MFCameraFormat, MFControl, MFFrameFormat, MFResolution,
         MediaFoundationControls, MediaFoundationDeviceDescriptor,
     };
-    use std::ffi::c_void;
     use std::{
         borrow::Cow,
         cell::Cell,
+        ffi::c_void,
         mem::MaybeUninit,
         slice::from_raw_parts,
         sync::{
@@ -369,7 +383,9 @@ pub mod wmf {
     };
     use windows::{Guid, Interface};
 
-    static mut INITIALIZED: Arc<AtomicBool> = Arc::new(AtomicBool::new(false));
+    lazy_static! {
+        static ref INITIALIZED: Arc<AtomicBool> = Arc::new(AtomicBool::new(false));
+    }
 
     // See: https://stackoverflow.com/questions/80160/what-does-coinit-speed-over-memory-do
     const CO_INIT_APARTMENT_THREADED: COINIT = COINIT(0x2);
@@ -395,7 +411,7 @@ pub mod wmf {
     const CAM_CTRL_MANUAL: i32 = 0x0002;
 
     pub fn initialize_mf() -> Result<(), BindingError> {
-        if !(unsafe { INITIALIZED.load(Ordering::SeqCst) }) {
+        if !(INITIALIZED.load(Ordering::SeqCst)) {
             let a = std::ptr::null_mut();
             if let Err(why) =
                 unsafe { CoInitializeEx(a, CO_INIT_APARTMENT_THREADED | CO_INIT_DISABLE_OLE1DDE) }
@@ -409,16 +425,13 @@ pub mod wmf {
                 }
                 return Err(BindingError::InitializeError(why.to_string()));
             }
-
-            unsafe {
-                INITIALIZED.store(true, Ordering::SeqCst);
-            }
+            INITIALIZED.store(true, Ordering::SeqCst);
         }
         Ok(())
     }
 
     pub fn de_initialize_mf() -> Result<(), BindingError> {
-        if unsafe { INITIALIZED.load(Ordering::SeqCst) } {
+        if INITIALIZED.load(Ordering::SeqCst) {
             unsafe {
                 if let Err(why) = MFShutdown() {
                     return Err(BindingError::DeInitializeError(why.to_string()));
@@ -783,12 +796,18 @@ pub mod wmf {
             let media_source = unsafe {
                 let mut receiver: MaybeUninit<IMFMediaSource> = MaybeUninit::uninit();
                 let mut ptr_receiver = receiver.as_mut_ptr();
-                self.source_reader.GetServiceForStream(
+                if let Err(why) = self.source_reader.GetServiceForStream(
                     MEDIA_FOUNDATION_FIRST_VIDEO_STREAM,
                     &MF_MEDIASOURCE_SERVICE,
                     &IMFMediaSource::IID,
                     &mut ptr_receiver as *mut *mut IMFMediaSource as *mut *mut c_void,
-                );
+                ) {
+                    return Err(BindingError::GUIDSetError(
+                        "MEDIA_FOUNDATION_FIRST_VIDEO_STREAM".to_string(),
+                        "MF_MEDIASOURCE_SERVICE".to_string(),
+                        why.to_string(),
+                    ));
+                }
                 receiver.assume_init()
             };
 
@@ -1282,12 +1301,18 @@ pub mod wmf {
             let media_source = unsafe {
                 let mut receiver: MaybeUninit<IMFMediaSource> = MaybeUninit::uninit();
                 let mut ptr_receiver = receiver.as_mut_ptr();
-                self.source_reader.GetServiceForStream(
+                if let Err(why) = self.source_reader.GetServiceForStream(
                     MEDIA_FOUNDATION_FIRST_VIDEO_STREAM,
                     &MF_MEDIASOURCE_SERVICE,
                     &IMFMediaSource::IID,
                     &mut ptr_receiver as *mut *mut IMFMediaSource as *mut *mut c_void,
-                );
+                ) {
+                    return Err(BindingError::GUIDSetError(
+                        "MEDIA_FOUNDATION_FIRST_VIDEO_STREAM".to_string(),
+                        "MF_MEDIASOURCE_SERVICE".to_string(),
+                        why.to_string(),
+                    ));
+                }
                 receiver.assume_init()
             };
 
